/*
 * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */


package org.graalvm.compiler.jtt.bytecode;

import org.junit.Test;

import org.graalvm.compiler.jtt.JTTTest;

/*
 */
public class BC_lcmp extends JTTTest {

    public static boolean test(long a, long b) {
        return a < b;
    }

    @Test
    public void run0() throws Throwable {
        runTest("test", 0L, -1L);
    }

    @Test
    public void run1() throws Throwable {
        runTest("test", 77L, 78L);
    }

    @Test
    public void run2() throws Throwable {
        runTest("test", -1L, 0L);
    }

    /**
     * Test with ugly numbers (which probably does not fit into one instruction.
     *
     * @throws Throwable
     */
    @Test
    public void run3() throws Throwable {
        runTest("test", 293521900824L, 97726785831L);
    }

    /**
     * Test with big numbers where it makes difference if the value is handled with 64 bits.
     *
     * @throws Throwable
     */
    @Test
    public void run4() throws Throwable {
        runTest("test", -1L, Long.MIN_VALUE);
    }

}
